/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTSocket.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- inlude files:
-------------------------------------------------------------------*/
#include <BTSocket.h>
#include <iostream>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string>
using namespace std;


//*******************************************************************
// BTSocket                                              CONSTRUCTOR
//*******************************************************************
BTSocket::BTSocket()
: fd_( INVALID_SOCKET )
{}

BTSocket::BTSocket( const int in_fd )
: fd_ ( in_fd )
{}

BTSocket::BTSocket( const Domain in_domain )
: fd_( INVALID_SOCKET )
{
   int domain;

   switch( in_domain ) {
      case UNIX_DOMAIN:
         domain = AF_LOCAL;
         break;
      case IPv4:
         domain = AF_INET;
         break;
      case IPv6:
         domain = AF_INET6;
         break;
      default:
         domain = UNKNOWN_DOMAIN;
   }

   if( domain != UNKNOWN_DOMAIN ) {
      fd_ = ::socket( domain, SOCK_STREAM, 0 );
      if( INVALID_SOCKET == fd_ ) {
          throw BTSocketException( string( strerror( errno ) ) );
      }
   }
}
// end of BTSocket

//*******************************************************************
// close                                                      PUBLIC
//*******************************************************************
void BTSocket::close()
{
   if( fd_ != INVALID_SOCKET ) {
      ::close( fd_ );
   }
   fd_ = INVALID_SOCKET;
}
// end of close

//*******************************************************************
// read                                                      PRIVATE
//*******************************************************************
bool BTSocket::read( string& out_buffer, const int in_nbytes )
{
   bool retval = false;

   out_buffer.clear();
   if( in_nbytes < BUFFER_SIZE ) {
      char buffer[ BUFFER_SIZE ];
      memset( buffer, 0, BUFFER_SIZE );
      retval = ( in_nbytes == read( buffer, in_nbytes ) );
      if( retval ) {
         out_buffer = string( buffer );
      }
   }
   return retval;
}
int BTSocket::read( void* const out_buffer, const int in_nbytes )
{
   char* ptr   = static_cast< char* >( out_buffer );
   int   nleft = in_nbytes;

   while( nleft > 0 ) {
      const int nread = ::read( fd_, ptr, nleft );
      //-------------------------------------------------------
      if( nread < 0 ) {
         if( EINTR == errno ) continue;
         else                 throw BTSocketException( string( strerror( errno ) ) );
      }
      else if( 0 == nread ) {
         break;
      }
      //-------------------------------------------------------
      nleft -= nread;
      ptr   += nread;
   }

   return ( in_nbytes - nleft );
}
// end of read

//*******************************************************************
// write                                                     PRIVATE
//*******************************************************************
bool BTSocket::write( const string& in_buffer )
{
   const int n = in_buffer.length();
   return ( n == write( in_buffer.c_str(), n ) );
}
int BTSocket::write( const void* const in_buffer, const int in_nbytes )
{
   const char* ptr   = static_cast< const char* >( in_buffer );
   int         nleft = in_nbytes;

   while( nleft > 0 ) {
      const int nwritten = ::write( fd_, ptr, nleft );
      //-------------------------------------------------------
      if( nwritten <= 0 ) {
         if( EINTR == errno ) continue;
         else                 throw BTSocketException( string( strerror( errno ) ) );
      }
      //-------------------------------------------------------
      nleft -= nwritten;
      ptr   += nwritten;
   }

   return in_nbytes;
}
// end of write

//*******************************************************************
// listen                                                     PUBLIC
//*******************************************************************
void BTSocket::listen()
{
   if( ::listen( fd_, 1024 ) ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
}
// end of listen
