/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : ScopeDefs.cpp
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_ScopeDefs_h
#define INCLUDED_ScopeDefs_h

/*------- include files:
-------------------------------------------------------------------*/
#include <string>
#include <map>
#include <functional>


// ####### ScopeInfo ######################################

class ScopeInfo
{
// ******* CONSTRUCTION / DESTRUCTION *******
public:
   ScopeInfo()
   : class_( "" ), method_( "" ) {
   }
   ScopeInfo( const std::string& in_class, const std::string& in_method )
   : class_( in_class ), method_( in_method ) {
   }
   ScopeInfo( const ScopeInfo& rhs ) {
      class_  = rhs.class_;
      method_ = rhs.method_;
   }
   virtual ~ScopeInfo() {
   }
   ScopeInfo& operator=( const ScopeInfo& rhs ) {
      if( this != &rhs ) {
         class_  = rhs.class_;
         method_ = rhs.method_;
      }
      return *this;
   }
   bool operator< ( const ScopeInfo& rhs ) const {
      return ( class_ == rhs.class_ ) ? ( method_ < rhs.method_ ) : ( class_ < rhs.class_ );
   }

// ******* MEMBERS *******
public:
   std::string   class_;
   std::string   method_;
};

// ####### ScopeDefs ############################

class ScopeDefs
{
private:
   class Sorter
   {
   public:
      bool operator() ( const ScopeInfo& in_si_1, const ScopeInfo& in_si_2 ) const {
         return in_si_1 < in_si_2;
      }
   };
// ******* TYPES *******
public:
   typedef std::map< ScopeInfo, unsigned int, Sorter > ItemsMap;

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   ScopeDefs() {}
   virtual ~ScopeDefs() {}
private:
   ScopeDefs( const ScopeDefs& );
   ScopeDefs& operator=( const ScopeDefs& );

// ******* MEMBERS *******
private:
   static unsigned int id_counter_;
   ItemsMap info_;

// ******* METHODS *******
public:
   void clear() {
      info_.clear();
   }
   const ItemsMap& scopes() const {
      return info_;
   }
   bool exists( const ScopeInfo& in_scope ) {
      ItemsMap::const_iterator it = info_.find( in_scope );
      return ( it != info_.end() );
   }
   int scope_id( const std::string& in_class, const std::string& in_method ) {
      int retval = -1;
      ItemsMap::const_iterator it = info_.find( ScopeInfo( in_class, in_method ) );
      if( it != info_.end() ) {
         retval = it->second;
      }
      return retval;
   }
   void add( const ScopeInfo& in_scope ) {
      if( info_.find( in_scope ) == info_.end() ) {
         info_[ in_scope ] = id_counter_++;
      }
   }
   void add( const ScopeInfo& in_scope, const unsigned long in_id ) {
      if( info_.find( in_scope ) == info_.end() ) {
         info_[ in_scope ] = in_id;
      }
   }
   void print_info() const;
};

// ####### InvScopeDefs ############################

class InvScopeDefs
{
// ******* TYPES *******
public:
   typedef std::map<unsigned int, ScopeInfo > InvItemsMap;

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   InvScopeDefs() {}
   virtual ~InvScopeDefs() {}
private:
   InvScopeDefs( const InvScopeDefs& );
   InvScopeDefs& operator=( const InvScopeDefs& );

// ******* MEMBERS *******
private:
   InvItemsMap info_;
   InvItemsMap::const_iterator it_;

// ******* METHODS *******
public:
   const InvItemsMap::const_iterator first() {
      return ( it_ = info_.begin() );
   }
   const InvItemsMap::const_iterator next() {
      return ++it_;
   }
   const InvItemsMap::const_iterator end() const {
      return info_.end();
   }
   void add( const unsigned int in_id, const ScopeInfo& in_scope ) {
      if( info_.find( in_id ) == info_.end() ) {
         info_[ in_id ] = in_scope;
      }
   }
   bool scope( const unsigned int in_id, ScopeInfo& out_scope ) {
      bool retval = false;
      InvItemsMap::const_iterator it = info_.find( in_id );
      if( it != info_.end() ) {
         out_scope = it->second;
         retval = true;
      }
      return retval;
   }
};

#endif // INCLUDED_ScopeDefs_h
