/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Tracer.cpp
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_Trace_h
#define INCLUDED_Trace_h

#define BEESOFT_TRACER

/*------- include files:
-------------------------------------------------------------------*/
#include <BTMutex.h>
#include <iostream>
#include <string>
#include <sstream>
#include <stdio.h>

/*------- forward declarations:
-------------------------------------------------------------------*/
class BTUnixSocket;

/*------- class declarations:
-------------------------------------------------------------------*/

class Tracer
{
// ******* TYPES *******
public:
   enum {
      INVALID_TYPE = 0,
      MSG,           // 1
      WARNING,       // 2
      ERROR,         // 3
      SCOPE_IN,      // 4
      SCOPE_OUT,     // 5
      SCOPE_DEF,     // 6
      RESET,         // 7
      END_MARKER
   };

// ******* CONSTRUCTION / DESTRUCTION *******
private:
   Tracer();
   virtual ~Tracer();
   Tracer( const Tracer& );
   Tracer& operator=( const Tracer& );

// ******* CONSTANTS *******
private:
   static const char* const UNIX_DEV;
public:
   static const std::string HEADER[];
   
 // ******* MEMBERS *******
private:
   static Tracer* tracer_;
   BTUnixSocket*  socket_;
   BTMutex        mutex_;

// ******* METHODS *******
public:
   static Tracer* const instance();
   static bool    is_valid_type ( int );
   void send( int, const std::string&, const std::string&, const std::string& );
   void send( int, const std::string&, const std::string& );
};

/**
 * TraceSender - obiekty tworzone makrami DBG_MSG, DBG_WARNING i DBG_ERROR
 */
template< int N >
class TraceSender
{
public:
   TraceSender( const std::string& in_class, const std::string& in_method )
   : class_ ( in_class )
   , method_( in_method )
   {
      buffer_.clear();
   }
   virtual ~TraceSender()
   {}
   TraceSender( const TraceSender& rhs ) {
      buffer_.str( rhs.buffer_.str() );
      class_ = rhs.class_;
      method_ = rhs.method_;
   }
   TraceSender& operator=( const TraceSender& rhs ) {
      if( this != &rhs ) {
         buffer_.str( rhs.buffer_.str() );
         class_ = rhs.class_;
         method_ = rhs.method_;
      }
      return *this;
   }
   template< typename T >
   TraceSender<N>& operator<< ( const T in_arg ) {
      buffer_ << in_arg;
      if( buffer_.str().find_last_of( '\n' ) != std::string::npos ) {
         Tracer::instance()->send( N, class_, method_, buffer_.str() );
      }
      return *this;
   }
private:
   std::ostringstream buffer_;
   std::string        class_;
   std::string        method_;
};

/**
 * TraceScope - obiekt tworzony przez makro TRC_SCOPE
 */
class TraceScope
{
public:
   TraceScope( const std::string& in_class, const std::string& in_method )
   : class_( in_class )
   , method_( in_method )
   {
      Tracer::instance()->send( Tracer::SCOPE_IN, class_, method_ );
   }
   virtual ~TraceScope()
   {
      Tracer::instance()->send( Tracer::SCOPE_OUT, class_, method_ );
   }
   TraceScope( const TraceScope& rhs ) {
      class_  = rhs.class_;
      method_ = rhs.method_;
   }
   TraceScope& operator=( const TraceScope& rhs ) {
      if( this != &rhs ) {
         class_  = rhs.class_;
         method_ = rhs.method_;
      }
      return *this;
   }
   template< int N >
   TraceSender<N> sender() {
      return TraceSender<N>( class_, method_ );
   }
private:
   std::string        class_;
   std::string        method_;
};

/**
 * KLASA TraceScopeDef - obiekt tworzony przez makro TRC_SCOPE_DEF
 */
class TraceScopeDef
{
public:
   TraceScopeDef( const std::string& in_class, const std::string& in_method )
   : class_  ( in_class  )
   , method_ ( in_method )
   {
      Tracer::instance()->send( Tracer::SCOPE_DEF, class_, method_ );
   }
   virtual ~TraceScopeDef()
   {}
   TraceScope scope( const std::string& in_class, const std::string& in_method ) {
      return TraceScope( in_class, in_method );
   }
private:
   TraceScopeDef( const TraceScopeDef& );
   TraceScopeDef& operator=( const TraceScopeDef& );
private:
   const std::string class_;
   const std::string method_;
};

/*------- macros:
-------------------------------------------------------------------*/
#ifdef BEESOFT_TRACER
   #define TRC_SCOPE_DEF( class_name, method_name )  \
   namespace bsc##class_name##method_name \
   { \
      TraceScopeDef trace_scope_def( #class_name, #method_name ); \
   }
#else
   #define TRC_SCOPE_DEF( p1, p2 )
#endif

#ifdef BEESOFT_TRACER
   #define TRC_SCOPE( class_name, method_name ) \
   TraceScope trace_scope = bsc##class_name##method_name::trace_scope_def.scope( #class_name, #method_name )
#else
   #define TRC_SCOPE( p1, p2 ) \
   std::ostringstream dev_null
#endif

#ifdef BEESOFT_TRACER
   #define DBG_MSG trace_scope.sender<Tracer::MSG>()
#else
   #define DBG_MSG dev_null
#endif

#ifdef BEESOFT_TRACER
   #define DBG_WARNING trace_scope.sender<Tracer::WARNING>()
#else
   #define DBG_WARNING dev_null
#endif

#ifdef BEESOFT_TRACER
   #define DBG_ERROR  trace_scope.sender<Tracer::ERROR>()
#else
   #define DBG_ERROR dev_null
#endif

#endif // INCLUDED_Tracer_h
