/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : TraceClient.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTCallArgs.h>
#include <BTTcpSocket.h>
#include <ScopeDefs.h>
#include <Tracer.h>
#include <iostream>
#include <iomanip>
#include <string>
#include <vector>
#include <set>
using namespace std;

/*------- local variables:
-------------------------------------------------------------------*/
static ScopeDefs scopes;
static InvScopeDefs inv_scopes;


//*******************************************************************
// display_help
//*******************************************************************
void display_help()
{
   cout << endl << "Trace Client is part of Beesoft Tracer" << endl << endl;

   cout << "Using:" << endl;
   cout << "\ttracecli [-port <port_number>] [-ip <address_ip>]" << endl<< endl;
   cout << "where <address_ip> is address of Trace Server," << endl;
   cout << "and <port_number> is a port on which server waits for connection." << endl;
   cout << "Default values are: address: 127.0.0.1, port 6789." << endl;
   cout << "For help (this text) type 'tracesrv' width --help or -h." << endl;

   cout << endl;
}
// end of display_help


//*******************************************************************
// read_next
//*******************************************************************
static bool read_next( const int in_type )
{
   return !( ( Tracer::SCOPE_IN == in_type ) || ( Tracer::SCOPE_OUT == in_type ) );
}
// end of read_next

//*******************************************************************
// display_error
//*******************************************************************
static void display_error( const int in_size )
{
   cerr << "ERROR: message length exceed buffer length (" << in_size << ")" << endl;
}
// end of display_error

//*******************************************************************
// display
//*******************************************************************
static void display( const int in_type, const int in_id )
{
   ScopeInfo si;
   if( inv_scopes.scope( in_id, si ) ) {
      cout << setw( 10 ) << left << Tracer::HEADER[ in_type ]
         << ": (" <<  si.class_ << "," << si.method_ << ") ";
         if( Tracer::SCOPE_IN == in_type ) cout << ">>>>";
         else                              cout << "<<<<";
         cout << endl;
   }
}
static void display( const int in_type, const int in_id, const string& in_buffer )
{
   ScopeInfo si;
   if( inv_scopes.scope( in_id, si ) ) {
      cout  << setw( 10 ) << left
            << Tracer::HEADER[ in_type ]
            << ": (" <<  si.class_ << "," << si.method_ << ") "
            << in_buffer
            << endl;
   }
}
// end of display

//*******************************************************************
// split
//*******************************************************************
static void split( const string& in_text, vector<string>& out_data )
{
   out_data.clear();
   
   unsigned int pos1 = 0;
   unsigned int pos2 = in_text.find( '|' );

   while( pos2 != string::npos ) {
      out_data.push_back( in_text.substr( pos1, pos2 - pos1 ) );
      pos1 = pos2 + 1;
      pos2 = in_text.find( '|', pos1 );
   }

   out_data.push_back( in_text.substr( pos1 ) );
}
// end of split

//*******************************************************************
// add_scope
//-------------------------------------------------------------------
// Zapamietujemy informacje o scope.
// Sa to: nazwa klasy, nazwa funkcji i numer id tej pary.
// Pozniej skorzystamy z tego, gdy beda przychodzily komunikaty.
// Wraz z komunikatem przyjdzie numer id tej pary.
// Dzieki temu bedziemy mogli wyswietlic nazwe klasy i funkcji,
// do ktorych odnosi sie ten komunikat.
//*******************************************************************
static void add_scope( const int in_id, const string& in_text )
{
   vector<string> items;
   
   split( in_text, items );
   if( 2 == items.size() ) {
      ScopeInfo si( items[ 0 ], items[ 1 ] );
      if( !scopes.exists( si ) ) {
         scopes.add( si, in_id );
         inv_scopes.add( in_id, si );
      }
   }
}
// add_scope

//*******************************************************************
// read_data
//*******************************************************************
static void read_data( BTTcpSocket& in_socket )
{
   int type, id, n;
   string text;

   if( in_socket.read<int>( type ) ) {
      if( in_socket.read<int>( id ) ) {
         if( read_next( type ) ) {
            if( in_socket.read<int>( n ) ) {
               if( in_socket.read( text, n ) ) {
                  if( Tracer::SCOPE_DEF == type ) {
                     add_scope( id, text );
                  }
                  else {
                     display( type, id, text );
                  }
               }
            }
         }
         else {
            display( type, id );
         }
      }
   }
}
// end of read_data

//*******************************************************************
// main
//*******************************************************************
int main( const int n, const char** const args )
{
   BTCallArgs pars( n, args );

   if( pars.need_help() ) {
      display_help();
   }
   else if( pars.port_is_valid() && pars.addr_is_valid() ) {
      BTTcpSocket socket( pars.addr(), pars.port() );

      try {
         socket.connect();
         while( true ) {
            read_data( socket );
         }
         socket.close();
         cout << "OK. Finish." << endl;
      }
      catch( BTSocketException& in_exception ) {
         cerr << "Exit on error: " << in_exception.what() << endl;
         socket.close();
      }
   }
   else {
      cerr << "Invalid call parameters (port or address)" << endl;
   }
   
   return 0;
}
// end of main
