/* TraceClear.cpp */

/*------- include files:
-------------------------------------------------------------------*/
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
using namespace std;

/*------- local types:
-------------------------------------------------------------------*/
typedef std::vector<std::string> Files;

enum {
   ERROR = -1,
   IS_UNKNOWN,
   IS_DIR,
   IS_FILE
};

/*------- variables:
-------------------------------------------------------------------*/
const int BUFFER_SIZE = 1024;
char buffer[ BUFFER_SIZE ];

/*------- forward declarations:
-------------------------------------------------------------------*/
void do_item( const string& );


//*******************************************************************
// need_help
//*******************************************************************
bool need_help( const Files& in_files )
{
   bool retval = false;

   for( int i = 0; i < in_files.size(); ++i ) {
      if( ( in_files[ i ] == "--help" ) || ( in_files[ i ] == "-h" ) ) {
         retval = true;
         break;
      }
   }
   return retval;
}
// end of need_help

//*******************************************************************
// display_help
//*******************************************************************
void display_help()
{
   cout << endl << "Trace Cleaner is part of Beesoft Tracer" << endl;
   cout << "Trace cleaner removes from source files all trace commands." << endl << endl;
   cout << "Using:" << endl;
   cout << "\ttraceclr par1 par2 par3 ....." << endl<< endl;
   cout << "where par1, par2 .... are paths to source files and/or directories where are this files." << endl;
   cout << "For help (this text) type 'traceclr' without parameters or width --help or -h." << endl;

   cout << endl;
}
// end of display_help

//*******************************************************************
// valid_extension
//*******************************************************************
bool valid_extension( const string& in_fpath )
{
   static const string extensions[] = { "c", "cc", "cpp", "cxx", "h", "hh", "hpp", "hxx" };
   bool retval = false;
   
   const string::size_type idx = in_fpath.rfind( '.' );
   if( idx != string::npos ) {
      if( idx != 0 ) {
         const string ext = in_fpath.substr( idx + 1 );
         const int    n   = sizeof( extensions ) / sizeof( extensions[ 0 ] );
         for( int i = 0; i < n; ++i ) {
            if( extensions[ i ] == ext ) {
               retval = true;
               break;
            }
         }
      }
   }
   return retval;
}
// end of valid_extension

//*******************************************************************
// prepare_src_file
//*******************************************************************
bool prepare_src_file( const string& in_fpath, const string& in_bak_fpath )
{
   remove( in_bak_fpath.c_str() );
   return ( 0 == rename( in_fpath.c_str(), in_bak_fpath.c_str() ) );
}
// end of prepare_src_file

//*******************************************************************
// restore_src_file
//*******************************************************************
bool restore_src_file( const string& in_bak_fpath, const string& in_fpath )
{
   remove( in_fpath.c_str() );
   return ( 0 == rename( in_bak_fpath.c_str(), in_fpath.c_str() ) );
}
// end of restore_src_file

//*******************************************************************
// remove_text
//*******************************************************************
void remove_text( string& inout_text, const string& in_muster )
{
   for(;;) {
      const string::size_type idx = inout_text.find( in_muster );
      if( string::npos == idx ) break;
      const string::size_type idx2 = inout_text.find( ';', idx );
      if( string::npos == idx2 ) break;
      inout_text.erase( idx, idx2 - idx + 1 );
   }
}
// end of remove_text

//*******************************************************************
// parse_and_copy
//*******************************************************************
bool parse_and_copy( ifstream& in_src, ofstream& in_dst )
{
   static const string TrcScopeDef = "TRC_SCOPE_DEF";
   static const string TrcScope    = "TRC_SCOPE";
   static const string DbgMsg      = "DBG_MSG";
   static const string DbgWarning  = "DBG_WARNING";
   static const string DbgError    = "DBG_ERROR";

   bool retval = true;
   
   memset( buffer, 0, BUFFER_SIZE );
   while( in_src.getline( buffer, BUFFER_SIZE ) ) {
      if( in_src.fail() ) {
         retval = false;
         break;
      }
      
      // parse and copy ---------------
      string line( buffer );
      remove_text( line, TrcScopeDef );
      remove_text( line, TrcScope );
      remove_text( line, DbgMsg );
      remove_text( line, DbgWarning );
      remove_text( line, DbgError );
      in_dst << line << endl;
      //-------------------------------

      if( in_src.eof() ) {
         break;
      }
   }
   return retval;
}
// end of parse_and_copy

//*******************************************************************
// check_item_type
//*******************************************************************
int check_item_type( const string& in_fpath )
{
   int retval = ERROR;

   struct stat attr;
   if( 0 == stat( in_fpath.c_str(), &attr ) ) {
           if( S_ISDIR( attr.st_mode ) ) retval = IS_DIR;
      else if( S_ISREG( attr.st_mode ) ) retval = IS_FILE;
      else                               retval = IS_UNKNOWN;
   }
   return retval;
}
// end of check_item_type

//*******************************************************************
// parse_dir
//*******************************************************************
void parse_dir( const string& in_fpath )
{
   Files files;
   
   DIR* dir = opendir( in_fpath.c_str() );
   if( dir ) {
      struct dirent* fent;
      while( ( fent = readdir( dir ) ) != 0 ) {
         if( fent->d_name[ 0 ] != '.' ) {
            string file_path = in_fpath;
            if( in_fpath != "/" ) file_path += "/";
            file_path += fent->d_name;
            files.push_back( file_path );
         }
      }
      closedir( dir );
   }

   for_each( files.begin(), files.end(), do_item );
}
// end of parse_dir

//*******************************************************************
// parse_file
//*******************************************************************
void parse_file( const string& in_fpath )
{
   cout << "Parsing: " << in_fpath;

   const string bak_fpath = in_fpath + ".bak";
   if( prepare_src_file( in_fpath, bak_fpath ) ) {
      ifstream src( bak_fpath.c_str() );
      if( src.is_open() ) {
         ofstream dst( in_fpath.c_str() );
         if( dst.is_open() ) {
            if( parse_and_copy( src, dst ) ) {
               remove( bak_fpath.c_str() );
            }
            dst.close();
         }
         else {
            restore_src_file( bak_fpath, in_fpath );
            cout << "- Can't create new version of the file";
         }
         src.close();
      }
      else {
         cout << "- Can't open the file";
      }
   }
   else {
      cout << "- Can't rename the file";
   }
   
   cout << "." << endl;
}
// end of parse_file

//*******************************************************************
// do_item
//*******************************************************************
void do_item( const string& in_fpath )
{
   switch( check_item_type( in_fpath ) ) {
      case IS_DIR:
         parse_dir( in_fpath );
         break;
      case IS_FILE:
         if( valid_extension( in_fpath ) ) {
            parse_file( in_fpath );
         }
         break;
      default:
         cout << "Error: " << in_fpath << "is not a directory/file (or not found)." << endl;
   }
}
// end of do_item

//*******************************************************************
// main
//*******************************************************************
int main( const int n, const char** const pars )
{
   if( n > 1 ) {
      Files files;
      files.assign( pars + 1, pars + n );

      if( need_help( files ) ) display_help();
      else for_each( files.begin(), files.end(), do_item );
   }
   else {
      display_help();
   }
   return 0;
}
// end of main
