/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : EventsController.cpp
 * DATE   : 3.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "EventsController.h"
#include "Event.h"
#include <QApplication>
#include <QMutexLocker>

/*------- local static variables:
-------------------------------------------------------------------*/
EventsController* EventsController::d_instance = 0;


//*******************************************************************
// EventsController                              CONSTRUCTOR private
//*******************************************************************
EventsController::EventsController()
: QObject()
{
	setParent( qApp );
}
// end of EventsControler

//*******************************************************************
// ~EventsContoller                               DESTRUCTOR private
//*******************************************************************
EventsController::~EventsController()
{
	QMutexLocker locker( &d_mutex );
	d_items.clear();
}
// end of ~EventsController

//*******************************************************************
// instance                                                   PUBLIC
//*******************************************************************
EventsController* EventsController::instance()
{
	if( 0 == d_instance ) {
		d_instance = new EventsController;
	}
	return d_instance;
}
// end of instance

//*******************************************************************
// append                                                     PUBLIC
//-------------------------------------------------------------------
// Zarejestrowanie chetnego do odbierania wskazanego sygnalu.
//*******************************************************************
bool EventsController::append( QObject* const in_receiver, const qint32 in_event_id )
{
	QMutexLocker locker( &d_mutex );
	bool retval = true;

	if( exists( in_receiver, in_event_id ) ) {
		retval = false;
	}
	else {
		d_items[ in_event_id ] << in_receiver;
	}

	return retval;
}
// end of append

//*******************************************************************
// exists                                                    PRIVATE
//*******************************************************************
bool EventsController::exists( QObject* const in_receiver, int const in_event_id )
{
//	QMutexLocker locker( &d_mutex );  !!! ma tak zostac
	bool retval = false;

	if( d_items.contains( in_event_id ) ) {
		retval = d_items.value( in_event_id ).contains( in_receiver );
	}
	return retval;
}
// end of exists

//*******************************************************************
// send_event                                                 PUBLIC
//*******************************************************************

// WERSJA BEZ DANYCH
void EventsController::send_event( const qint32 in_event_id )
{
	QMutexLocker locker( &d_mutex );

	if( d_items.capacity() != d_items.size() ) {
		d_items.squeeze();
	}

	if( d_items.contains( in_event_id ) ) {
		const QSet< QObject* >& values = d_items[ in_event_id ];
		QSet< QObject* >::const_iterator it = values.begin();
		const QSet< QObject* >::const_iterator end = values.end();

		while( it != end ) {
			QApplication::postEvent( *it, new Event( in_event_id ) );
			++it;
		}
	}
}
// WERSJA Z 1 DANA
void EventsController::send_event( const qint32 in_event_id, const QVariant in_val )
{
	QMutexLocker locker( &d_mutex );

	if( d_items.capacity() != d_items.size() ) {
		d_items.squeeze();
	}

	if( d_items.contains( in_event_id ) ) {
		QSet< QObject* >& values = d_items[ in_event_id ];
		QSet< QObject* >::const_iterator it = values.begin();
		const QSet< QObject* >::const_iterator end = values.end();

		while( it != end ) {
			QApplication::postEvent( *it, new Event( in_event_id, in_val ) );
			++it;
		}
	}
}
// WERSJA Z 2 DANYMI
void EventsController::send_event( const qint32 in_event_id, const QVariant in_val1, const QVariant in_val2 )
{
	QMutexLocker locker( &d_mutex );

	if( d_items.capacity() != d_items.size() ) {
		d_items.squeeze();
	}

	if( d_items.contains( in_event_id ) ) {
		QSet< QObject* >& values = d_items[ in_event_id ];
		QSet< QObject* >::const_iterator it = values.begin();
		const QSet< QObject* >::const_iterator end = values.end();

		while( it != end ) {
			QApplication::postEvent( *it, new Event( in_event_id, in_val1, in_val2 ) );
			++it;
		}
	}
}
// WERSJA Z 3 DANYMI
void EventsController::send_event( const qint32 in_event_id, const QVariant in_val1, const QVariant in_val2, const QVariant in_val3 )
{
	QMutexLocker locker( &d_mutex );

	if( d_items.capacity() != d_items.size() ) {
		d_items.squeeze();
	}

	if( d_items.contains( in_event_id ) ) {
		QSet< QObject* >& values = d_items[ in_event_id ];
		QSet< QObject* >::const_iterator it = values.begin();
		const QSet< QObject* >::const_iterator end = values.end();

		while( it != end ) {
			QApplication::postEvent( *it, new Event( in_event_id, in_val1, in_val2, in_val3 ) );
			++it;
		}
	}
}
// WERSJA Z 4 DANYMI
void EventsController::send_event( const qint32 in_event_id, const QVariant in_val1, const QVariant in_val2, const QVariant in_val3, const QVariant in_val4 )
{
	QMutexLocker locker( &d_mutex );

	if( d_items.capacity() != d_items.size() ) {
		d_items.squeeze();
	}

	if( d_items.contains( in_event_id ) ) {
		QSet< QObject* >& values = d_items[ in_event_id ];
		QSet< QObject* >::const_iterator it = values.begin();
		const QSet< QObject* >::const_iterator end = values.end();

		while( it != end ) {
			QApplication::postEvent( *it, new Event( in_event_id, in_val1, in_val2, in_val3, in_val4 ) );
			++it;
		}
	}
}
// end of send_event

//*******************************************************************
// remove                                                     PUBLIC
//*******************************************************************
void EventsController::remove( QObject* const in_receiver )
{
	QMutexLocker locker( &d_mutex );

	if( !d_items.empty() ) {
		const QList< qint32 > ids = d_items.keys();
		QList< qint32 >::const_iterator it = ids.begin();
		const QList< qint32 >::const_iterator end = ids.end();
		while( it != end ) {
			d_items[ *it ].remove( in_receiver );
			++it;
		}
	}
	d_items.squeeze();
}
// end of remove
