/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : MainWindow.cpp
 * DATE   : 29.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "MainWindow.h"
#include "Settings.h"
#include "MessageBase.h"
#include "Table.h"
#include "Tree.h"
#include "Network.h"
#include "Monitor.h"
#include "ServersDialog.h"
#include "EventsController.h"
#include "Shared.h"
//..................
#include <QDockWidget>
#include <QMenu>
#include <QAction>
#include <QMenuBar>
#include <QStatusBar>
#include <QAction>
#include <QIcon>
#include <QApplication>
#include <QToolBar>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const MainWindow::TREE_TITLE              = QT_TR_NOOP( "Scopes view" );
const char* const MainWindow::DEFAULT_STATUS          = QT_TR_NOOP( "Ready..."    );
const char* const MainWindow::MENU_SYSTEM_LABEL       = QT_TR_NOOP( "&System"     );
const char* const MainWindow::MENU_TRACE_LABEL        = QT_TR_NOOP( "&Trace"      );
const char* const MainWindow::MENU_HELP_LABEL         = QT_TR_NOOP( "&Help"       );
const char* const MainWindow::ACTION_SERVER_LABEL     = QT_TR_NOOP( "&Server"     );
const char* const MainWindow::ACTION_CONNECT_LABEL    = QT_TR_NOOP( "&Connect"    );
const char* const MainWindow::ACTION_DISCONNECT_LABEL = QT_TR_NOOP( "&Disconnect" );
const char* const MainWindow::ACTION_RUN_LABEL        = QT_TR_NOOP( "&Run" );
const char* const MainWindow::ACTION_EXIT_LABEL       = QT_TR_NOOP( "&Exit"       );
const char* const MainWindow::ACTION_EXIT_KEY         = QT_TR_NOOP( "Ctrl+Q"      );


//*******************************************************************
// MainWindow                                            CONSTRUCTOR
//*******************************************************************
MainWindow::MainWindow() : QMainWindow()
, menu_system_       ( new QMenu )
, menu_trace_        ( new QMenu )
, menu_help_         ( new QMenu )
, toolbar_           ( new QToolBar )
, action_server_     ( new QAction( QIcon( ":/img/computer.png"   ), "", this ) )
, action_connect_    ( new QAction( QIcon( ":/img/connect.png"    ), "", this ) )
, action_disconnect_ ( new QAction( QIcon( ":/img/disconnect.png" ), "", this ) )
, action_run_        ( new QAction( QIcon( ":/img/go.png"         ), "", this ) )
, action_exit_       ( new QAction( QIcon( ":/img/exit.png"       ), "", this ) )
, dock_              ( new QDockWidget )
, table_             ( new Table )
, tree_              ( new Tree )
, network_           ( new Network )
, monitor_           ( new Monitor( table_ ) )
{
   setWindowTitle( Shared::full_program_name() );
   setCentralWidget( table_ );

   action_disconnect_->setEnabled( false );
   action_run_->setEnabled( false );
   dock_->toggleViewAction()->setIcon( QIcon( ":/img/dock.png"       ) );
   
   translate();
   create_menu();
   toolbar();

   toolbar_->setObjectName( "toolbar" );
   dock_->setObjectName( "docker" );
   
   dock_->setAllowedAreas( Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea );
   dock_->setWindowTitle( tr( TREE_TITLE ) );
   dock_->setWidget( tree_ );
   addDockWidget( Qt::LeftDockWidgetArea, dock_ );

   set_status( tr( DEFAULT_STATUS ) );
   update_actions();

   EventsController::instance()->append( this, Event::CONNECTED );
   EventsController::instance()->append( this, Event::DISCONNECTED );
   EventsController::instance()->append( this, Event::SOCKET_STATE );
}
// end of MainWindow

//*******************************************************************
// ~MainWindow                                            DESTRUCTOR
//*******************************************************************
MainWindow::~MainWindow()
{
   EventsController::instance()->remove( this );
  
   if( network_ ) {
      network_->disconnect();
      network_->join();
      delete network_;
   }
   if( monitor_ ) {
      monitor_->stop();
      monitor_->join();
      delete monitor_;
   }
   save_settings();
}
// end of ~MainWindow

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void MainWindow::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<qint32>( event->type() );
         
   switch( type ) {
      case Event::CONNECTED:
         action_connect_->setEnabled( false );
         action_disconnect_->setEnabled( true );
         action_run_->setEnabled( true );
         break;
      case Event::DISCONNECTED:
         action_connect_->setEnabled( true );
         action_disconnect_->setEnabled( false );
         action_run_->setEnabled( false );
         break;
      case Event::SOCKET_STATE:
         set_status( event->d_data[ 0 ].toString() );
         break;
   }
}
// end of customEvent

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void MainWindow::showEvent( QShowEvent* const in_event )
{
   Settings stt;
   QVariant data;

   if( stt.read( "MainWindow/Size", data ) ) {
      resize( data.toSize() );
   }
   else {
      Shared::resize( this, 60, 95 );
   }
   if( stt.read( "MainWindow/Pos", data ) ) {
      move( data.toPoint() );
   }

   restore_settings();
   QMainWindow::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// save_settings                                             PRIVATE
//*******************************************************************
void MainWindow::save_settings()
{
   Settings stt;
   stt.save( "MainWindow/State"  , saveState()        );
   stt.save( "MainWindow/Pos"    , pos()              );
   stt.save( "MainWindow/Size"   , size()             );
   stt.save( "CurrentServer/Name", Shared::srv_name() );
   stt.save( "CurrentServer/Addr", Shared::srv_addr() );
   stt.save( "CurrentServer/Port", Shared::srv_port() );
}
// end of save_settings

//*******************************************************************
// restore_settings                                          PRIVATE
//*******************************************************************
void MainWindow::restore_settings()
{
   Settings stt;
   QVariant data;

   if( stt.read( "MainWindow/State", data ) ) {
      restoreState( data.toByteArray() );
   }
   if( stt.read( "CurrentServer/Name", data ) ) {
      Shared::srv_name( data.toString() );
   }
   if( stt.read( "CurrentServer/Addr", data ) ) {
      Shared::srv_addr( data.toString() );
   }
   if( stt.read( "CurrentServer/Port", data ) ) {
      Shared::srv_port( data.toString() );
   }
}
// end of restore_settings

//*******************************************************************
// set_status                                                PRIVATE
//*******************************************************************
void MainWindow::set_status( const QString& in_text )
{
   statusBar()->showMessage( in_text );
}
// end of set_status

//*******************************************************************
// update_actions                                            PRIVATE
//*******************************************************************
void MainWindow::update_actions()
{
   connect( action_server_    , SIGNAL( triggered() ), this, SLOT( server_slot    () ) );
   connect( action_connect_   , SIGNAL( triggered() ), this, SLOT( connect_slot   () ) );
   connect( action_disconnect_, SIGNAL( triggered() ), this, SLOT( disconnect_slot() ) );
   connect( action_run_       , SIGNAL( triggered() ), this, SLOT( run_slot       () ) );
   connect( action_exit_      , SIGNAL( triggered() ), qApp, SLOT( closeAllWindows() ) );
}
// end of update_actions

//*******************************************************************
// create_menu                                               PRIVATE
//*******************************************************************
void MainWindow::create_menu()
{
   menuBar()->addMenu( menu_system_ );
   menuBar()->addMenu( menu_trace_  );
   menuBar()->addMenu( menu_help_   );

   menu_system_->addAction( dock_->toggleViewAction() );
   menu_system_->addSeparator();
   menu_system_->addAction( action_exit_ );

   menu_trace_->addAction( action_server_ );
   menu_trace_->addSeparator();
   menu_trace_->addAction( action_connect_ );
   menu_trace_->addAction( action_disconnect_ );
   menu_trace_->addSeparator();
   menu_trace_->addAction( action_run_ );
}
// end of create_menu

//*******************************************************************
// toolbar                                                   PRIVATE
//*******************************************************************
void MainWindow::toolbar()
{
   addToolBar( toolbar_ );
   toolbar_->addAction( action_exit_ );
   toolbar_->addSeparator();
   toolbar_->addAction( dock_->toggleViewAction() );
   toolbar_->addSeparator();
   toolbar_->addAction( action_server_ );
   toolbar_->addAction( action_connect_ );
   toolbar_->addAction( action_disconnect_ );
   toolbar_->addAction( action_run_ );
}
// end of toolbar

//*******************************************************************
// translate                                                 PRIVATE
//*******************************************************************
void MainWindow::translate()
{
   menu_system_->setTitle( tr( MENU_SYSTEM_LABEL ) );
   menu_trace_ ->setTitle( tr( MENU_TRACE_LABEL  ) );
   menu_help_  ->setTitle( tr( MENU_HELP_LABEL   ) );

   action_exit_->setText    ( tr( ACTION_EXIT_LABEL ) );
   action_exit_->setShortcut( tr( ACTION_EXIT_KEY   ) );
   
   action_server_    ->setText( tr( ACTION_SERVER_LABEL     ) );
   action_connect_   ->setText( tr( ACTION_CONNECT_LABEL    ) );
   action_disconnect_->setText( tr( ACTION_DISCONNECT_LABEL ) );
   action_run_       ->setText( tr( ACTION_RUN_LABEL        ) );
}
// end of translate

//*******************************************************************
// server_slot                                          PRIVATE slot
//*******************************************************************
void MainWindow::server_slot()
{
   ServersDialog* const sd = new ServersDialog( this );
   if( sd ) {
      sd->exec();
      delete sd;
   }
}
// end of server_slot

//*******************************************************************
// connect slot                                         PRIVATE slot
//*******************************************************************
void MainWindow::connect_slot()
{
   network_->connect_host( Shared::srv_addr().toStdString(), Shared::srv_port().toInt() );
}
// end of connect_slot

//*******************************************************************
// disconnect_slot                                      PRIVATE slot
//*******************************************************************
void MainWindow::disconnect_slot()
{
   MessageBase::instance()->stop();
   network_->disconnect();
   monitor_->stop();
}
// end of disconnect_slot

//*******************************************************************
// run                                                  PRIVATE slot
//*******************************************************************
void MainWindow::run_slot()
{
   EventsController::instance()->send_event( Event::GET_SELECTED_SCOPES );
}
// end of run_slot
