/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Table.h
 * DATE   : 29.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Table.h"
#include "TableItem.h"
#include "EventsController.h"
#include "Shared.h"
#include <Tracer.h>
#include <QHeaderView>
#include <QtDebug>

//*******************************************************************
// Table                                                 CONSTRUCTOR
//*******************************************************************
Table::Table( QWidget* const in_parent ) : QTableWidget( in_parent )
, icon_in_   ( QPixmap( ":/img/in.png"      ) )
, icon_out_  ( QPixmap( ":/img/out.png"     ) )
, icon_info_ ( QPixmap( ":/img/info.png"    ) )
, icon_warn_ ( QPixmap( ":/img/warning.png" ) )
, icon_err_  ( QPixmap( ":/img/error.png"   ) )
, x_width_   ( fontMetrics().width( "X" )   )
, x_height_  ( fontMetrics().height()       )
{
   mutex_.set_name( "Table" );
   
   setColumnCount( 4 );
   setEditTriggers( NoEditTriggers );
   setSelectionMode( SingleSelection );
   setSelectionBehavior( SelectRows );
   setAlternatingRowColors( true );

   verticalHeader()->hide();
   horizontalHeader()->setDefaultAlignment( Qt::AlignLeft );

   QStringList hdrtxt;
   hdrtxt << "class" << "function" << "" << "message";
   setHorizontalHeaderLabels ( hdrtxt );
   resize_columns();
   
   EventsController::instance()->append( this, Event::DISPLAY_MESSAGE );
}
// end of Table

//*******************************************************************
// resizeEvent                                     PRIVATE inherited
//*******************************************************************
void Table::resizeEvent( QResizeEvent* const in_event )
{
   QTableWidget::resizeEvent( in_event );
   resize_columns();
}
// end of resizeEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void Table::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<qint32>( event->type() );
         
   switch( type ) {
      case Event::DISPLAY_MESSAGE:
         display( event );
         break;
   }
}
// end of customEvent

//*******************************************************************
// display                                                   PRIVATE
//*******************************************************************
void Table::display( Event* const in_event )
{
   mutex_.lock();
   
   const qint32  type        = in_event->d_data[ 0 ].toInt();
   const QString class_name  = in_event->d_data[ 1 ].toString();
   const QString method_name = in_event->d_data[ 2 ].toString();
   const QString text        = in_event->d_data[ 3 ].toString();
   add( type, class_name, method_name, text );
   
   mutex_.unlock();
   Shared::idle();
}
// end of display

//*******************************************************************
// add                                                        PUBLIC
//*******************************************************************
void Table::add(  const qint32 in_type,
                  const QString& in_class,
                  const QString& in_method,
                  const QString& in_msg )
{
   const qint32 row = rowCount();
   insertRow( row );

   setItem( row, 0, new TableItem( in_class  ) );
   setItem( row, 1, new TableItem( in_method ) );
   const QPixmap* icon;
   switch( in_type ) {
      case Tracer::SCOPE_IN:
         icon = &icon_in_;
         break;
      case Tracer::SCOPE_OUT:
         icon = &icon_out_;
         break;
      case Tracer::MSG:
         icon = &icon_info_;
         break;
      case Tracer::WARNING:
         icon = &icon_warn_;
         break;
      case Tracer::ERROR:
         icon = &icon_err_;
         break;
   }
   TableItem* const new_item = new TableItem( QIcon( *icon ) );
   new_item->setTextAlignment( Qt::AlignHCenter );
   setItem( row, 2, new_item );
   
   setItem( row, 3, new TableItem( in_msg ) );
   setRowHeight( row, x_height_ + 2 );
   resize_columns();
   
   scrollToItem( item( row, 0 ) );
   setCurrentCell( row, 0 );
}
// end of add

//*******************************************************************
// resize_columns                                            PRIVATE
//*******************************************************************
void Table::resize_columns()
{
   resizeColumnToContents( 0 );
   resizeColumnToContents( 1 );
   setColumnWidth( 2, icon_in_.width() + 8 );

   const qint32 view_width = viewport()->width();
   const qint32 total_width = total_columns_width();
   if( total_width < view_width ) {
      setColumnWidth( 3, view_width - ( total_width - columnWidth( 3 ) ) );
   }
}
// end of resize_columns

//*******************************************************************
// total_columns_width                                       PRIVATE
//*******************************************************************
qint32 Table::total_columns_width() const
{
   qint32 retval = 0;
   const qint32 n = columnCount();

   for( qint32 i = 0; i < n; ++i ) {
      retval += columnWidth( i );
   }
   return retval;
}
// end of total_columns_width
