/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Tree.cpp
 * DATE   : 29.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Tree.h"
#include "TreeItem.h"
#include "EventsController.h"
#include "Scopes.h"
#include <ScopeDefs.h>
#include <QHeaderView>
#include <set>
#include <QtDebug>
using namespace std;

//*******************************************************************
// Tree                                                  CONSTRUCTOR
//*******************************************************************
Tree::Tree( QWidget* const in_parent ) : QTreeWidget( in_parent )
, check_state_( true )
{
   setColumnCount( 1 );
   header()->hide();
   setAlternatingRowColors( true );

   connect( this, SIGNAL( itemChanged ( QTreeWidgetItem*, int ) ),
            this, SLOT  ( item_changed( QTreeWidgetItem*      ) ) );
   
   EventsController::instance()->append( this, Event::SCOPES_READY );
   EventsController::instance()->append( this, Event::GET_SELECTED_SCOPES );
}
// end of Tree

//*******************************************************************
// ~Tree                                                  DESTRUCTOR
//*******************************************************************
Tree::~Tree()
{
   EventsController::instance()->remove( this );
}
// end of ~Tree

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void Tree::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<qint32>( event->type() );
         
   switch( type ) {
      case Event::SCOPES_READY:
         set_scopes();
         break;
      case Event::GET_SELECTED_SCOPES:
         get_selected_scopes();
         break;
   }
}
// end of customEvent

//*******************************************************************
// set_scopes                                                PRIVATE
//*******************************************************************
void Tree::set_scopes()
{
   qint32 id;
   ScopeInfo si;
   if( Scopes::instance()->first( id, si ) ) {
      add( id, QString( si.class_.c_str() ), QString( si.method_.c_str() ) );
      while( Scopes::instance()->next( id, si ) ) {
         add( id, QString( si.class_.c_str() ), QString( si.method_.c_str() ) );
      }
   }
   sortItems( 0, Qt::AscendingOrder );
}
// end of set_scopes

//*******************************************************************
// get_selected_scopes                                       PRIVATE
//*******************************************************************
void Tree::get_selected_scopes()
{
   set<int> selected_items;
   
   const QList<QTreeWidgetItem*> items = findItems( "*", Qt::MatchWildcard, 0 );
   if( !items.empty() ) {
      QList<QTreeWidgetItem*>::const_iterator it = items.begin();
      const QList<QTreeWidgetItem*>::const_iterator end = items.end();
      while( it != end ) {
         const TreeItem* const item = dynamic_cast<TreeItem*>( *it );
         if( Qt::Unchecked != item->checkState( 0 ) ) {
            const qint32 n = item->childCount();
            for( qint32 i = 0; i < n; ++i ) {
               const TreeItem* const child = dynamic_cast<TreeItem*>( item->child( i ) );
               if( Qt::Checked == child->checkState( 0 ) ) {
                  selected_items.insert( child->data( 0, ID ).toInt() );
               }
            }
         }
         ++it;
      }
   }
   if( !selected_items.empty() ) {
      Scopes::instance()->selected( selected_items );
      EventsController::instance()->send_event( Event::SCOPES_SELECTED );
   }
}
// end of get_selected_scopes

//*******************************************************************
// add                                                       PRIVATE
//*******************************************************************
void Tree::add( const qint32 in_id, const QString& in_class_name, const QString& in_method_name )
{
   TreeItem* const item = new TreeItem;
   if( item ) {
      TreeItem* parent = search_class( in_class_name );
      if( !parent ) {
         parent = add_class( in_class_name );
      }
      if( parent ) {
         item->setText( 0, in_method_name );
         item->setData( 0, TYPE, METHOD );
         item->setData( 0, ID, in_id );
         parent->addChild( item );
      }
   }
}
// end of add

//*******************************************************************
// add_class                                                 PRIVATE
//*******************************************************************
TreeItem* const Tree::add_class( const QString& in_class_name )
{
   TreeItem* const item = new TreeItem;
   if( item ) {
      item->setText( 0, in_class_name );
      item->setData( 0, TYPE, CLASS );
      item->setData( 0, ID, -1 );
      addTopLevelItem( item );
      item->setExpanded( true );
      header()->resizeSections( QHeaderView::ResizeToContents );
   }
   return item;
}
// end of add_class

//*******************************************************************
// search_class                                               PUBLIC
//*******************************************************************
TreeItem* const Tree::search_class ( const QString& in_class ) const
{
   TreeItem* retval = 0;

   const QList<QTreeWidgetItem*> items = findItems( in_class, Qt::MatchFixedString, 0 );
   if( !items.empty() ) {
      QList<QTreeWidgetItem*>::const_iterator it = items.begin();
      const QList<QTreeWidgetItem*>::const_iterator end = items.end();
      while( it != end ) {
         TreeItem* const item = dynamic_cast<TreeItem*>( *it );
         if( CLASS == item->data( 0, TYPE ).toInt() ) {
            retval = item;
            break;
         }
         ++it;
      }
   }
   return retval;
}
// end of search_class

//*******************************************************************
// item_changed                                         PRIVATE slot
//*******************************************************************
void Tree::item_changed( QTreeWidgetItem* in_item )
{
   if( check_state_ ) {
      check_state_ = false;
      TreeItem* const item = dynamic_cast<TreeItem*>( in_item );
      if( item ) {
         if( CLASS == item->data( 0, TYPE ) ) {
            const qint32 n = item->childCount();
            const Qt::CheckState state = item->status();
            for( qint32 i = 0; i < n; ++i ) {
               dynamic_cast<TreeItem*>( item->child( i ) )->status( state );
            }
         }
         else {
            TreeItem* const parent = dynamic_cast<TreeItem*>( item->parent() );
            parent->status( check_class_state( parent ) );
         }
      }
      check_state_ = true;
   }
}
// end of item_changed

//*******************************************************************
// check_class_state                                         PRIVATE
//*******************************************************************
Qt::CheckState Tree::check_class_state( const TreeItem* const in_parent ) const
{
   Qt::CheckState state = Qt::Unchecked;

   if( in_parent ) {
      const qint32 n = in_parent->childCount();
      if( n > 0 ) {
         const TreeItem* const item = dynamic_cast<TreeItem*>( in_parent->child( 0 ) );
         state = item->status();
         for( qint32 i = 1; i < n; ++i ) {
            const TreeItem* const item = dynamic_cast<TreeItem*>( in_parent->child( i ) );
            if( item->status() != state ) {
               state = Qt::PartiallyChecked;
               break;
            }
         }
      }
   }
   return state;
}
// end of check_class_state
